
# q-expansion of newform 588.3.g.e, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 588
weight = 3
poly_data = [4, -52, 187, -105, 67, 112, -38, -14, 37, -15, 8, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-289820015, 562744392, -283995830, 240576118, 356867930, -136366971, -42783443, 125089835, -50193231, 26569152, -10139764, 3413147], 41035869], [[-116729128, 494426674, -311973684, 170216824, 295355887, -98205199, -39234122, 96372196, -39240946, 20930237, -7839183, 2579651], 13678623], [[274623172, -1450104180, 850526839, -600856103, -889891150, 307820913, 109964215, -302830021, 122076888, -65063823, 24539699, -8240656], 41035869], [[28244696, -227166732, 110501483, -101827915, -146269259, 51928002, 9674783, -45937001, 18155502, -10058658, 3725380, -1333499], 5862267], [[652818209, -3175449228, 1635954827, -1493961451, -1918270070, 701915490, 172474748, -650451800, 258878601, -140318385, 52661629, -18513815], 41035869], [[46209352, -319049625, 176388739, -158264519, -198000436, 74166318, 16376476, -65283331, 25889703, -14154870, 5339045, -1870213], 3908178], [[526603612, -3733476936, 1910849110, -1763247110, -2257390837, 819485655, 187015858, -745012024, 295244970, -161455677, 60487529, -21305845], 41035869], [[-4923788336, 25900697571, -15227936291, 10637452771, 16046669612, -5814733506, -1667294864, 5177314517, -2087562795, 1126277676, -425583769, 144053501], 82071738], [[1957924013, -14395824072, 7446798914, -6857831878, -8635172030, 3194951769, 681280181, -2840172329, 1125694737, -615906996, 231615892, -81643325], 41035869], [[1418627216, -10321376471, 5621861151, -5224530623, -6464913032, 2443551926, 496215148, -2125672397, 842041247, -461191804, 173955153, -61404289], 27357246], [[1623954904, -11308626077, 6253602755, -5625097563, -7038922864, 2667486032, 556373430, -2316115645, 919240997, -502597738, 189819541, -66759459], 27357246]]

hecke_ring_character_values = [[295, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [197, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [493, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 2, 0, 0, -1, 2, 0, 0, 0, 0, 0, 0], [0, 0, 0, 3, 0, 0, 0, 0, -1, 0, 0, -1], [0, 0, -3, -1, 0, 0, 0, 0, 0, 0, 2, -2], [0, 0, -2, 1, 0, 0, 0, 0, 1, 0, -2, -1], [0, 5, 0, 0, -1, 4, 0, -6, 0, -1, 0, 0], [2, 3, 0, 0, 0, -2, 0, 6, 0, -1, 0, 0], [0, 0, 6, -1, 0, 0, 4, 0, -1, 0, -2, -3], [-6, 1, 0, 0, 0, 2, 0, 10, 0, -3, 0, 0], [0, 0, -5, -5, 0, 0, 0, 0, -2, 0, -6, 4], [0, 0, 0, 0, -4, -4, 0, -4, 0, -4, 0, 0], [0, 0, -4, 2, 0, 0, 16, 0, 2, 0, -4, -2], [6, -5, 0, 0, 0, 6, 0, -2, 0, -1, 0, 0], [0, 0, 12, -2, 0, 0, 4, 0, -2, 0, -4, -6], [0, 0, 6, 5, 0, 0, 0, 0, 3, 0, 8, -5], [0, 0, 0, 0, -10, -8, 0, -4, 0, -8, 0, 0], [0, 7, 0, 0, 5, 12, 0, -2, 0, 5, 0, 0], [0, 0, -18, -2, 0, 0, 0, 0, -4, 0, 4, -8], [0, -6, 0, 0, 10, -8, 0, -20, 0, -2, 0, 0], [0, 0, 20, -6, 0, 0, 20, 0, -6, 0, 4, -2], [0, 0, 7, -13, 0, 0, 0, 0, 6, 0, -2, 8], [0, 0, -30, -20, 0, 0, 0, 0, -2, 0, 4, -6], [0, 0, -21, 7, 0, 0, 0, 0, -8, 0, 4, -12], [0, 0, -6, 1, 0, 0, 12, 0, 1, 0, 2, 3], [0, -4, 0, 0, -5, 6, 0, 44, 0, 10, 0, 0], [14, 14, 0, 0, 0, -12, 0, 20, 0, -2, 0, 0], [74, 14, 0, 0, 0, -20, 0, -4, 0, 6, 0, 0], [0, -18, 0, 0, -14, -16, 0, 52, 0, 2, 0, 0], [0, 0, -8, 0, 0, 0, -20, 0, 0, 0, 8, 8], [-50, -8, 0, 0, 0, 8, 0, -8, 0, 0, 0, 0], [0, 0, 12, -2, 0, 0, 28, 0, -2, 0, -4, -6], [-58, 11, 0, 0, 0, -26, 0, -34, 0, 15, 0, 0], [0, 48, 0, 0, -8, 32, 0, -80, 0, -16, 0, 0], [0, 0, -18, -25, 0, 0, 0, 0, -3, 0, -16, 13], [0, 36, 0, 0, -22, 16, 0, -52, 0, -20, 0, 0], [0, 0, 4, 2, 0, 0, -48, 0, 2, 0, -12, -10], [0, 0, 7, 33, 0, 0, 0, 0, -10, 0, -4, -6], [0, -14, 0, 0, -19, -10, 0, 64, 0, 4, 0, 0], [0, 0, 24, 31, 0, 0, 0, 0, -5, 0, -8, 3], [0, 1, 0, 0, 21, 24, 0, 26, 0, 23, 0, 0], [-26, 13, 0, 0, 0, 6, 0, 70, 0, -19, 0, 0], [-10, 23, 0, 0, 0, -26, 0, 14, 0, 3, 0, 0], [0, 0, 8, -4, 0, 0, -88, 0, -4, 0, 8, 4], [0, -36, 0, 0, 40, -20, 0, 4, 0, 16, 0, 0], [0, 0, -4, 2, 0, 0, -24, 0, 2, 0, -4, -2], [0, 0, -20, 2, 0, 0, 84, 0, 2, 0, 12, 14], [0, 0, 12, -17, 0, 0, 0, 0, -1, 0, -32, 31], [206, 0, 0, 0, 0, -8, 0, -24, 0, 8, 0, 0], [0, 17, 0, 0, -37, -4, 0, -6, 0, -21, 0, 0], [0, 0, 30, 48, 0, 0, 0, 0, -2, 0, 12, -14], [0, 0, -32, 8, 0, 0, 36, 0, 8, 0, 0, 8], [0, 0, -1, 7, 0, 0, 0, 0, 2, 0, 14, -12], [0, -1, 0, 0, -21, -16, 0, -2, 0, -15, 0, 0], [0, 0, 41, 27, 0, 0, 0, 0, 2, 0, -8, 10], [0, 0, 14, -5, 0, 0, 60, 0, -5, 0, 6, 1], [166, 7, 0, 0, 0, 22, 0, 94, 0, -29, 0, 0], [70, 22, 0, 0, 0, -4, 0, 76, 0, -18, 0, 0], [0, 0, -30, 7, 0, 0, 136, 0, 7, 0, 2, 9], [0, 0, 3, -43, 0, 0, 0, 0, 18, 0, 8, 10], [0, 0, -30, 7, 0, 0, 104, 0, 7, 0, 2, 9], [0, 0, -52, 18, 0, 0, -136, 0, 18, 0, -20, -2], [0, 0, -72, -14, 0, 0, 0, 0, -6, 0, 40, -46], [-98, -27, 0, 0, 0, 26, 0, -30, 0, 1, 0, 0], [0, -12, 0, 0, -8, -36, 0, -44, 0, -24, 0, 0], [238, 0, 0, 0, 0, -8, 0, -24, 0, 8, 0, 0], [0, 28, 0, 0, 5, 42, 0, 4, 0, 14, 0, 0], [0, 0, 78, 11, 0, 0, 0, 0, 17, 0, -16, 33], [0, 0, 41, 1, 0, 0, 0, 0, 22, 0, 26, -4], [0, 13, 0, 0, -7, 16, 0, 10, 0, 3, 0, 0], [0, 0, -80, 24, 0, 0, -160, 0, 24, 0, -16, 8], [-86, 4, 0, 0, 0, 16, 0, 64, 0, -20, 0, 0], [0, -72, 0, 0, -8, -44, 0, 172, 0, 28, 0, 0], [0, 0, 32, -4, 0, 0, 184, 0, -4, 0, -16, -20], [-14, 19, 0, 0, 0, -34, 0, -26, 0, 15, 0, 0], [0, 0, 30, 63, 0, 0, 0, 0, 5, 0, 48, -43], [46, -32, 0, 0, 0, 8, 0, -104, 0, 24, 0, 0], [0, 0, 78, -16, 0, 0, 0, 0, 22, 0, -28, 50], [0, 0, -48, 12, 0, 0, 180, 0, 12, 0, 0, 12], [38, -9, 0, 0, 0, -18, 0, -90, 0, 27, 0, 0], [0, -23, 0, 0, 21, 8, 0, 74, 0, 31, 0, 0], [0, 0, -36, -78, 0, 0, 0, 0, 14, 0, 0, 14], [0, 0, 28, 2, 0, 0, -24, 0, 2, 0, -36, -34], [0, -68, 0, 0, 41, -22, 0, 124, 0, 46, 0, 0], [18, -68, 0, 0, 0, 72, 0, -56, 0, -4, 0, 0], [18, 20, 0, 0, 0, -8, 0, 56, 0, -12, 0, 0], [0, 0, 43, 35, 0, 0, 0, 0, -16, 0, -56, 40], [0, -6, 0, 0, -2, 16, 0, 76, 0, 22, 0, 0], [0, 0, -32, 16, 0, 0, 44, 0, 16, 0, -32, -16], [0, 0, -16, 4, 0, 0, -216, 0, 4, 0, 0, 4], [0, -12, 0, 0, -28, -32, 0, 8, 0, -20, 0, 0], [0, -2, 0, 0, -99, -58, 0, 32, 0, -56, 0, 0], [0, 12, 0, 0, 56, 12, 0, -124, 0, 0, 0, 0], [0, 0, -28, 6, 0, 0, -136, 0, 6, 0, 4, 10], [0, 0, -39, -35, 0, 0, 0, 0, -8, 0, -20, 12], [0, 0, -23, 39, 0, 0, 0, 0, -12, 0, 26, -38], [0, 0, -8, 4, 0, 0, -260, 0, 4, 0, -8, -4], [422, 75, 0, 0, 0, -90, 0, 30, 0, 15, 0, 0], [0, 84, 0, 0, -42, 56, 0, -84, 0, -28, 0, 0], [-298, 11, 0, 0, 0, -26, 0, -34, 0, 15, 0, 0], [0, 0, 52, -22, 0, 0, 300, 0, -22, 0, 36, 14], [310, -50, 0, 0, 0, 68, 0, 4, 0, -18, 0, 0], [0, -60, 0, 0, -22, -120, 0, -76, 0, -60, 0, 0], [0, 0, 60, 36, 0, 0, 0, 0, -8, 0, -48, 40], [0, 0, 12, 10, 0, 0, 316, 0, 10, 0, -52, -42], [0, 0, 37, 103, 0, 0, 0, 0, -36, 0, -42, 6], [0, -5, 0, 0, 1, 60, 0, 198, 0, 65, 0, 0], [0, 0, -12, 20, 0, 0, 0, 0, -32, 0, -64, 32], [0, 0, -68, 10, 0, 0, 344, 0, 10, 0, 28, 38], [278, 68, 0, 0, 0, -112, 0, -64, 0, 44, 0, 0], [-490, -18, 0, 0, 0, 4, 0, -60, 0, 14, 0, 0], [0, 0, 92, -34, 0, 0, 116, 0, -34, 0, 44, 10], [0, -42, 0, 0, 26, -72, 0, -100, 0, -30, 0, 0], [550, 70, 0, 0, 0, -52, 0, 124, 0, -18, 0, 0], [0, 0, 78, -15, 0, 0, 232, 0, -15, 0, -18, -33], [0, 0, -4, -2, 0, 0, -480, 0, -2, 0, 12, 10], [-222, -13, 0, 0, 0, 6, 0, -34, 0, 7, 0, 0], [0, -16, 0, 0, 119, 6, 0, -156, 0, 22, 0, 0], [0, 0, -42, 33, 0, 0, 0, 0, -49, 0, -72, 23], [282, 55, 0, 0, 0, -46, 0, 82, 0, -9, 0, 0], [0, 0, 35, -123, 0, 0, 0, 0, 54, 0, 4, 50], [0, 16, 0, 0, 53, 42, 0, -44, 0, 26, 0, 0], [0, 0, -16, -16, 0, 0, -212, 0, -16, 0, 80, 64], [226, -29, 0, 0, 0, 38, 0, -2, 0, -9, 0, 0], [222, -2, 0, 0, 0, 20, 0, 52, 0, -18, 0, 0], [0, 0, 8, -8, 0, 0, 216, 0, -8, 0, 24, 16], [0, 0, 138, -31, 0, 0, -52, 0, -31, 0, -14, -45], [0, 0, 48, -55, 0, 0, 0, 0, 5, 0, -88, 93], [0, 120, 0, 0, -30, 200, 0, 180, 0, 80, 0, 0], [0, -35, 0, 0, -43, -120, 0, -134, 0, -85, 0, 0], [0, -48, 0, 0, -120, -40, 0, 312, 0, 8, 0, 0], [318, -26, 0, 0, 0, 68, 0, 100, 0, -42, 0, 0], [0, 0, 63, 33, 0, 0, 0, 0, 4, 0, -18, 22], [0, 0, -180, -198, 0, 0, 0, 0, 14, 0, 24, -10], [0, 0, -133, -109, 0, 0, 0, 0, -20, 0, -36, 16], [0, 0, 20, -6, 0, 0, 500, 0, -6, 0, 4, -2], [0, 0, -109, 149, 0, 0, 0, 0, -50, 0, 108, -158], [170, -46, 0, 0, 0, -20, 0, -244, 0, 66, 0, 0], [0, 0, 64, -24, 0, 0, 380, 0, -24, 0, 32, 8], [-298, -25, 0, 0, 0, 46, 0, 38, 0, -21, 0, 0], [0, -174, 0, 0, 22, -168, 0, 148, 0, 6, 0, 0], [0, -238, 0, 0, 75, -142, 0, 376, 0, 96, 0, 0], [0, 0, -264, -123, 0, 0, 0, 0, -39, 0, 24, -63], [0, 0, 16, 8, 0, 0, -360, 0, 8, 0, -48, -40], [0, 0, -18, -57, 0, 0, 0, 0, -3, 0, -48, 45], [0, 0, 7, 101, 0, 0, 0, 0, -48, 0, -50, 2], [0, 0, 6, 13, 0, 0, -184, 0, 13, 0, -58, -45], [0, 143, 0, 0, -9, 128, 0, -170, 0, -15, 0, 0], [214, -144, 0, 0, 0, 120, 0, -216, 0, 24, 0, 0], [0, 0, 89, 243, 0, 0, 0, 0, -60, 0, -26, -34], [0, 24, 0, 0, 120, 20, 0, -276, 0, -4, 0, 0], [0, 0, -108, 42, 0, 0, 0, 0, 42, 0, -60, -18], [0, -156, 0, 0, 108, -52, 0, 252, 0, 104, 0, 0], [0, 157, 0, 0, -11, 176, 0, -78, 0, 19, 0, 0], [0, -108, 0, 0, 164, 32, 0, 200, 0, 140, 0, 0], [0, 0, -105, 131, 0, 0, 0, 0, -38, 0, 122, -160], [0, 0, -24, 148, 0, 0, 0, 0, -52, 0, 16, -68], [0, 0, 49, 111, 0, 0, 0, 0, 10, 0, 92, -82], [0, -166, 0, 0, 151, -38, 0, 248, 0, 128, 0, 0], [-590, -48, 0, 0, 0, -40, 0, -312, 0, 88, 0, 0], [0, 54, 0, 0, 66, 72, 0, -132, 0, 18, 0, 0], [0, 0, 52, -18, 0, 0, -380, 0, -18, 0, 20, 2], [1006, 28, 0, 0, 0, 8, 0, 136, 0, -36, 0, 0], [0, 0, 124, -18, 0, 0, -464, 0, -18, 0, -52, -70], [0, 180, 0, 0, 44, 248, 0, -64, 0, 68, 0, 0], [0, 0, 222, 53, 0, 0, 0, 0, 51, 0, -16, 67]]
